/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 * \brief This file contains API for IR distance Click driver.
 *
 * \addtogroup irdistance IR distance Click Driver
 * @{
 */
// ----------------------------------------------------------------------------

#ifndef IRDISTANCE_H
#define IRDISTANCE_H

#include "drv_digital_out.h"
#include "drv_digital_in.h"
#include "drv_analog_in.h"

// -------------------------------------------------------------- PUBLIC MACROS 
/**
 * \defgroup macros Macros
 * \{
 */

/**
 * \defgroup map_mikrobus MikroBUS
 * \{
 */
#define IRDISTANCE_MAP_MIKROBUS( cfg, mikrobus ) \
  cfg.out = MIKROBUS( mikrobus, MIKROBUS_AN ); \
  cfg.en  = MIKROBUS( mikrobus, MIKROBUS_RST )
/** \} */

/**
 * \defgroup error_code Error Code
 * \{
 */
#define IRDISTANCE_RETVAL  uint8_t

#define IRDISTANCE_OK           0x00
#define IRDISTANCE_INIT_ERROR   0xFF
/** \} */

/**
 * \defgroup various_registers Various Registers
 * \{
 */
#define IRDISTANCE_RESOLUTION_10_BIT           1024
#define IRDISTANCE_RESOLUTION_12_BIT           4096
#define IRDISTANCE_VCC_3_3                     3300.0
#define IRDISTANCE_VCC_5_0                     5000.0
/** \} */

/** \} */ // End group macro 
// --------------------------------------------------------------- PUBLIC TYPES
/**
 * \defgroup type Types
 * \{
 */
 
 /**
 * @brief Analog data type 
 */
typedef  analog_in_data_t irdistance_data_t;

/**
 * @brief Click ctx object definition.
 */
typedef struct
{
    // Output pins 

    digital_out_t en;

    // Modules 
    analog_in_t adc;

} irdistance_t;

/**
 * @brief Click configuration structure definition.
 */
typedef struct
{
    // Additional gpio pins 

    pin_name_t out;
    pin_name_t en;

    // Static variable 

    hal_adc_channel_t     channel;      // Channel
    hal_adc_resolution_t  resolution;   // Resolution
    hal_adc_voltage_reference_t vref;   // VRef    

} irdistance_cfg_t;

/** \} */ // End types group
// ----------------------------------------------- PUBLIC FUNCTION DECLARATIONS

/**
 * \defgroup public_function Public function
 * \{
 */
 
#ifdef __cplusplus
extern "C"{
#endif

/**
 * @brief Config Object Initialization function.
 *
 * @param cfg  Click configuration structure.
 *
 * @description This function initializes click configuration structure to init state.
 * @note All used pins will be set to unconnected state.
 */
void irdistance_cfg_setup ( irdistance_cfg_t *cfg );

/**
 * @brief Initialization function.
 * @param irdistance Click object.
 * @param cfg Click configuration structure.
 * 
 * @description This function initializes all necessary pins and peripherals used for this click.
 */
IRDISTANCE_RETVAL irdistance_init ( irdistance_t *ctx, irdistance_cfg_t *cfg );

/**
 * @brief Click Default Configuration function.
 *
 * @param ctx  Click object.
 *
 * @description This function executes default configuration for IR distance click.
 */
void irdistance_default_cfg ( irdistance_t *ctx );

/**
 * @brief Enable IR sensor function
 *
 * @param ctx  Click object.
 * 
 * @description Function enable Sharp�s GP2Y0A60SZ0F distance measuring sensor
 * by set RST pin on Stretch click board.
*/
void irdistance_enable ( irdistance_t *ctx );

/**
 * @brief Disable IR sensor function
 *
 * @param ctx  Click object.
 * 
 * @description Function disable Sharp�s GP2Y0A60SZ0F distance measuring sensor
 * by clear RST pin on Stretch click board.
*/
void irdistance_disable ( irdistance_t *ctx );

/**
 * @brief Generic read function.
 *
 * @param ctx        Click object.
 * @returns ADC data
 *
 * @description This function reads ADC data using analog_in_read function
 */
irdistance_data_t irdistance_read_adc ( irdistance_t *ctx );

/**
 * @brief Get the voltage output of the sensor function
 *
 * @param adc_value           The measurement adc value
 *
 * @param resolution_value    1024 : 10bit resolution 
 *                            4096 : 12bit resolution
 *
 * @param vcc_value           3300 : 3.3V VCC 
 *                            5000 : 5.0V VCC
 *
 * @returns float value of the sensor voltage output
 *
 * @description Function calculate the voltage output of the Sharp�s GP2Y0A60SZ0F 
 * distance measuring sensor on IR Distance click board.
*/
float irdistance_get_voltage_out ( irdistance_data_t adc_value, uint16_t resolution_value, float vcc_value );

#ifdef __cplusplus
}
#endif
#endif  // _IRDISTANCE_H_

/** \} */ // End public_function group
/// \}    // End click Driver group  
/*! @} */
// ------------------------------------------------------------------------- END
